;This file is copyright (c) 1991, 1992 Informant Communications Group and the
;article author. The material here may be used in an application provided
;that this copyright/disclaimer information is kept in the original source
;file. The material presented here is provided "as is" and with no guarantee.
;Informant Communications Group/Paradox Informant assume no responsibility
;for the use or misuse of the material contained within.
;
;Contents        : procedure GoDown(),
;                            GoRight(),
;                            MakeThisFieldCalulated(),
;                            MakeLinkedFieldsCalulated()
;
;Source File     : CALCLINK.SC
;Author          : David Faulkner
;                  Silver Software
;                  745-E Pulehuiki Rd.
;                  Kula, HI  96790
;                  (808) 878-2714
;
;Informant Issue : December 1992
;
;Description     : Set of procedures that converts "Regular" fields in 
;                  reports to "Calculated" fields to avoid problems should
;                  a table be restructured.
;
; Paradox Informant
; 10519 E. Stockton Blvd.
; Suite 142
; Elk Grove, CA  95624-9743
; Phone: (916) 686-6610
; Fax  : (916) 686-8497
; BBS  : (916) 686-4740

PROC GoDown()
  PRIVATE OldRowNo
  OldRowNo=ROWNO()
  DOWN
  IF OldRowNo=ROWNO() THEN
    RETURN FALSE              ;if cant go down must be on last row
  ELSE
    CTRLHOME                  ;leave cursor on first column
    RETURN TRUE
  ENDIF
ENDPROC

;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 

PROC GoRight()
  Private OldColNo
  OldColNo=COLNO()
  RIGHT
  IF OldColNo=COLNO() THEN
    ;must be at the end of a RowNo, so go to next one
    RETURN GoDown()
  ENDIF
  RETURN TRUE
ENDPROC

;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
Proc MakeThisFieldCalulated()
  Private FieldName,           ;Name of field the cursor is on
          Justify,             ;Type of justification for current field
          WordWrap,            ;Number of wordwrap lines for this field
          StartCol,            ;Column field starts at
          StartPage,           ;Page field starts at
          StartChar,           ;First character of field mask
          ReformatMenu,        ;MENUCHOICE from the reformat menu
          IsADateField,        ;True if this is a date field
          IsANumberField,      ;True if this is a numberic field
          EndDecimalCol,       ;Column field ends at
          EndDecimalPage,      ;Page field ends at
          Commas,              ;Comma format for numbers
          International        ;International format for numbers


  FieldName=FIELDINFO()

  Justify="Default"
  WordWrap="1"

  IF MATCH(fieldname,".., right..",a,b) THEN
    Justify="Right"
    fieldname=a+b
  ENDIF
  IF MATCH(fieldname,".., left..",a,b) THEN
    Justify="Left"
    fieldname=a+b
  ENDIF
  IF MATCH(fieldname,".., center..",a,b) THEN
    Justify="Center"
    fieldname=a+b
  ENDIF
  IF MATCH(fieldname,".., wrap:..",fieldname,b) THEN
    Wordwrap=b
  ENDIF

  StartCol=COLNO()
  StartPage=PAGENO()
  StartChar=CURSORCHAR()

  MENU {Field} {Reformat} ENTER
  ReformatMenu=MENUCHOICE()
  IsADateField =  ReformatMenu<>"" AND ReformatMenu<>"Digits"
  IsANumberField = ReformatMenu = "Digits"
  ENTER
  EndCol=COLNO()
  EndPage=PAGENO()
  IF IsANumberField THEN
    ENTER                     ;number of whole number places
    EndDecimalCol=COLNO()     ;number of decimal places
    EndDecimalPage=PAGENO()
    IF StartChar<>"9" THEN;could be ( or +- for sign convention
      EndDecimalCol=EndDecimalCol-1   ;sub on to account for extra character
      IF EndDecimalCol<0 THEN
        EndDecimalCol=PAGEWIDTH()-1
      ENDIF
      EndCol=EndCol-1
      IF EndCol<0 THEN
        EndCol=EndCol-1
      ENDIF
    ENDIF
    ;now need to pick up all the specks from the number
    MENU {Field} {Reformat} ENTER {Commas}        Commas=MENUCHOICE()
    MENU {Field} {Reformat} ENTER {International} International=MENUCHOICE()
  ENDIF

  MENU {Field} {Erase} ENTER
  WHILE NOT (COLNO()=StartCol AND PAGENO()=StartPage)
    LEFT
  ENDWHILE
  MENU {Field} {Place} {Calculated}
  TYPEIN FieldName
  ENTER   ;end of typein
  IF IsADateField THEN
    SELECT ReformatMenu          ;date format
    ENTER                 ;start of date
    LEFT                  ;put cursor on date
  ELSE
    ENTER   ;beginning of field
    IF IsANumberField THEN
      ;if this is a Dollar field, will already start as paren-negitive
      HOME  ;goto beginning of places field
      ;if beginning of field is not where we placed it then this is a
      ;dollar field, need to add one to account for the paren-negitive
      IF NOT (COLNO()=StartCol AND PAGENO()=StartPage) THEN
        EndCol=MOD(EndCol+1,PAGEWIDTH())
        EndDecimalCol=MOD(EndDecimalCol+1,PAGEWIDTH())
      ENDIF
      END   ;back to end of numberic field
    ENDIF
    WHILE NOT (COLNO()=EndCol AND PAGENO()=EndPage)
      LEFT
    ENDWHILE
    ENTER ;end of field
  ENDIF

  IF IsANumberField THEN;need to place digits
    WHILE NOT (COLNO()=EndDecimalCol AND PAGENO()=EndDecimalPage)
      RIGHT
    ENDWHILE

    ENTER
    MENU {Field} {Reformat} ENTER {International} SELECT International
    IF StartChar<>"9" THEN
      MENU {Field} {Reformat} ENTER {Sign-Convention}
      SELECT IIF(StartChar="(","ParenNegative","AlwaysSign")
      GoRight()  ;get to last char of field mask
    ENDIF
  ENDIF

  IF Justify<>"Default" THEN
    MENU {Field} {Justify} ENTER SELECT Justify
  ENDIF
  IF Wordwrap<>"1" THEN
    MENU {Field} {Wordwrap} ENTER SELECT Wordwrap
  ENDIF

EndProc

;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 

PROC MakeLinkedFieldsCalulated()
  PRIVATE TotalSpots,       ;Number of possible cursor positions on this report
          OldBand,          ;Name of last BAND DIVIDER cursor was on
          IsCalulated       ;Is current field a calulated field


IF SYSMODE()<>"Report" THEN
  RETURN "You must be in the report designer to use this script"
ENDIF

END
TotalSpots=ROWNO()*PAGEWIDTH()*NPAGES()/100
HOME        ;Take cursor to upper left hand corner
CTRLHOME    ;

IF ISINSERTMODE() THEN
  INS        ;get into typeover mode
ENDIF

OldBand=""
IF BANDINFO()="Report Header" THEN
  OldBand=BANDINFO()
ENDIF

WHILE TRUE
  IF COLNO()<>0 THEN
    MESSAGE INT(((ROWNO()-1)*NPAGES()*PAGEWIDTH()+
    (PAGENO()-1)*PAGEWIDTH()+
    COLNO())/TotalSpots),"%"
  ENDIF

  ;make sure we are not on a band
  IF OldBand<>BANDINFO() THEN
    IF NOT GoDown() THEN
      QUITLOOP
    ENDIF
    OldBand=BANDINFO()
    LOOP
  ENDIF

  IF CURSORCHAR()=" " THEN      ;find next non space cursorchar
    MENU {Field} {Place} {Calculated} {PI()} ENTER
    IF SEARCH("Not enough space",WINDOW())=0 THEN
      ENTER   ;end of place to put calculated field
      MENU {Field} {Erase} ENTER
    ELSE
      ;out side of table band, go down a row since nothing can be placed
      ;in this area.
      MENU ESC         ;Get out of calulated field placement mode
      IF GoDown() THEN
        LOOP
      ELSE
        QUITLOOP
      ENDIF
    ENDIF
  ENDIF


  ;only continue if on a linked, non summary field
  IF SEARCH("->",FIELDINFO())=0 OR SUBSTR(FIELDINFO(),1,1)<>"[" THEN
    IF GoRight() THEN
      LOOP
    ELSE
      QUITLOOP
    ENDIF
  ENDIF

  ;Check to see if we are sitting on a calulated field
  MENU {Field} {CalcEdit}
  ENTER
  IsCalulated=ISBLANK(WINDOW())
  MENU ESC
  IF IsCalulated THEN
    IF GoRight() THEN
      LOOP
    ELSE
      QUITLOOP
    ENDIF
  ENDIF

  ;At this point the cursor is on a linked field
  MakeThisFieldCalulated()

  IF NOT GoRight() THEN
    QUITLOOP
  ENDIF
ENDWHILE

EndProc

;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

MakeLinkedFieldsCalulated()
